#!/usr/bin/env python
# Copyright 2016, 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

# Workaround for missing Jinja2 import
import StringIO
import os

try:
    from jinja2 import FileSystemLoader, Environment

    TEMPLATE_ENV = Environment(loader=FileSystemLoader([os.path.dirname(__file__), '.']))


    def rewrite_haproxy(servers):
        '''Configure redis master monitoring'''
        ha_template = TEMPLATE_ENV.get_template('haproxy-cfg.tmpl')
        return StringIO.StringIO(ha_template.render(servers=servers))

except ImportError as ie:

    __PREAMBLE = '''
    defaults REDIS
    mode tcp
    log global
    option tcplog
    option dontlognull
    log /dev/log local0 err alert
    timeout connect 3s
    timeout server 6s
    timeout client 6s

#    frontend redis
#        bind *:6379 name master_redis
#        default_backend redis_backend
    
    backend redis_backend
        option tcp-check
        tcp-check send PING\\r\\n
        tcp-check expect string +PONG
        tcp-check send info\ replication\\r\\n
        tcp-check expect string role:master
        tcp-check send QUIT\\r\\n
        tcp-check expect string +OK
    '''

    __REPLICAS = '''
    backend replica_backend
        option tcp-check
        tcp-check send PING\r\n
        tcp-check expect string +PONG
        tcp-check send info\ replication\r\n
        tcp-check expect string role:slave
        tcp-check send QUIT\r\n
        tcp-check expect string +OK
    '''

    __SENTINELS = '''
    backend sentinel_backend
        option tcp-check
        tcp-check send PING\\r\\n
        tcp-check expect string +PONG
    '''

    __EPILOGUE = '''
    listen admin
	mode http
    	bind *:9001
    	stats enable
    	stats uri /
    '''


    def rewrite_haproxy(servers):
        hacfg = StringIO.StringIO()
        hacfg.write('%s\n' % __PREAMBLE )
        for server in servers:
            hacfg.write('        server m_{0} {0}:6379 maxconn 1024 check inter 1s\n'.format(server))
        hacfg.write('%s\n' % __REPLICAS)
        for server in servers:
            hacfg.write('        server r_{0} {0}:6379 maxconn 1024 check inter 1s\n'.format(server))
        hacfg.write('%s\n' % __SENTINELS)
        for server in servers:
            hacfg.write('        server s_{0} {0}:26379 maxconn 1024 check inter 1s\n'.format(server))
        hacfg.write('\n%s\n' % __EPILOGUE )
        return hacfg
